@@ -19,7 +19,6 @@ use rio_backend::sugarloaf::{
     Content, ContentBuilder, FragmentStyle, FragmentStyleDecoration, Graphic, Stretch,
     Style, SugarCursor, Sugarloaf, UnderlineInfo, UnderlineShape, Weight,
 };
-#[cfg(not(use_wa))]
 use rio_window::window::Theme;
 use std::collections::HashMap;
 use std::ops::RangeInclusive;
@@ -60,45 +59,20 @@ pub struct Renderer {
 }
 
 impl Renderer {
-    pub fn new(
-        #[cfg(not(use_wa))] config: &Config,
-        #[cfg(use_wa)] config: &Config,
-        #[cfg(not(use_wa))] current_theme: Option<Theme>,
-        #[cfg(use_wa)] appearance: wa::Appearance,
-    ) -> Renderer {
+    pub fn new(config: &Config, current_theme: Option<Theme>) -> Renderer {
         let term_colors = TermColors::default();
         let colors = List::from(&term_colors);
         let mut named_colors = config.colors;
 
-        #[cfg(not(use_wa))]
-        {
-            if let Some(theme) = current_theme {
-                if let Some(adaptive_colors) = &config.adaptive_colors {
-                    match theme {
-                        Theme::Light => {
-                            named_colors = adaptive_colors.light.unwrap_or(named_colors);
-                        }
-                        Theme::Dark => {
-                            named_colors = adaptive_colors.dark.unwrap_or(named_colors);
-                        }
-                    }
-                }
-            }
-        }
-
-        #[cfg(use_wa)]
-        {
+        if let Some(theme) = current_theme {
             if let Some(adaptive_colors) = &config.adaptive_colors {
-                match appearance {
-                    wa::Appearance::Light => {
+                match theme {
+                    Theme::Light => {
                         named_colors = adaptive_colors.light.unwrap_or(named_colors);
                     }
-                    wa::Appearance::Dark => {
+                    Theme::Dark => {
                         named_colors = adaptive_colors.dark.unwrap_or(named_colors);
                     }
-                    // TODO
-                    wa::Appearance::LightHighContrast => {}
-                    wa::Appearance::DarkHighContrast => {}
                 }
             }
         }
@@ -130,7 +104,7 @@ impl Renderer {
             is_vi_mode_enabled: false,
             is_blinking: false,
             last_typing: None,
-            config_has_blinking_enabled: config.blinking_cursor,
+            config_has_blinking_enabled: config.cursor.blinking,
             term_has_blinking_enabled: false,
             ignore_selection_fg_color: config.ignore_selection_fg_color,
             colors,
@@ -146,9 +120,9 @@ impl Renderer {
             dynamic_background,
             active_search: None,
             cursor: Cursor {
-                content: config.cursor,
-                content_ref: config.cursor,
-                state: CursorState::new(config.cursor),
+                content: config.cursor.shape.into(),
+                content_ref: config.cursor.shape.into(),
+                state: CursorState::new(config.cursor.shape.into()),
             },
             width_cache: FxHashMap::default(),
         }
@@ -230,14 +204,14 @@ impl Renderer {
             Some(background_color)
         };
 
-        let (decoration, decoration_color) = self.compute_decoration(square, false);
+        let (decoration, decoration_color) = self.compute_decoration(square);
 
         (
             FragmentStyle {
                 width,
                 color: foreground_color,
                 background_color,
-                font_attrs,
+                font_attrs: font_attrs.into(),
                 decoration,
                 decoration_color,
                 ..FragmentStyle::default()
@@ -246,23 +220,21 @@ impl Renderer {
         )
     }
 
+    #[inline]
     fn compute_decoration(
         &self,
         square: &Square,
-        skip_underline: bool,
     ) -> (Option<FragmentStyleDecoration>, Option<[f32; 4]>) {
         let mut decoration = None;
         let mut decoration_color = None;
 
         if square.flags.contains(Flags::UNDERLINE) {
-            if !skip_underline {
-                decoration = Some(FragmentStyleDecoration::Underline(UnderlineInfo {
-                    offset: -1.0,
-                    size: 1.0,
-                    is_doubled: false,
-                    shape: UnderlineShape::Regular,
-                }));
-            }
+            decoration = Some(FragmentStyleDecoration::Underline(UnderlineInfo {
+                offset: -1.0,
+                size: 1.0,
+                is_doubled: false,
+                shape: UnderlineShape::Regular,
+            }));
         } else if square.flags.contains(Flags::STRIKEOUT) {
             decoration = Some(FragmentStyleDecoration::Strikethrough);
         } else if square.flags.contains(Flags::DOUBLE_UNDERLINE) {
@@ -332,26 +304,6 @@ impl Renderer {
                     self.create_style(square)
                 };
 
-            if square.flags.contains(Flags::GRAPHICS) {
-                // let graphics = square.graphics().map(|graphics| {
-                //     graphics
-                //         .iter()
-                //         .map(|graphic| Graphic {
-                //             id: graphic.texture.id,
-                //             offset_x: graphic.offset_x,
-                //             offset_y: graphic.offset_y,
-                //         })
-                //         .collect::<_>()
-                // });
-                // style.media = Some(graphics);
-                let graphic = &square.graphics().unwrap()[0];
-                style.media = Some(Graphic {
-                    id: graphic.texture.id,
-                    offset_x: graphic.offset_x,
-                    offset_y: graphic.offset_y,
-                });
-            }
-
             if self.hyperlink_range.is_some()
                 && square.hyperlink().is_some()
                 && self
@@ -397,9 +349,33 @@ impl Renderer {
                 }
             }
 
+            if square.flags.contains(Flags::GRAPHICS) {
+                // let graphics = square.graphics().map(|graphics| {
+                //     graphics
+                //         .iter()
+                //         .map(|graphic| Graphic {
+                //             id: graphic.texture.id,
+                //             offset_x: graphic.offset_x,
+                //             offset_y: graphic.offset_y,
+                //         })
+                //         .collect::<_>()
+                // });
+                // style.media = Some(graphics);
+                let graphic = &square.graphics().unwrap()[0];
+                style.media = Some(Graphic {
+                    id: graphic.texture.id,
+                    offset_x: graphic.offset_x,
+                    offset_y: graphic.offset_y,
+                });
+                style.background_color = None;
+            }
+
             if last_style != style {
                 if !content.is_empty() {
+                    // let start = std::time::Instant::now();
                     content_builder.add_text(&content, last_style);
+                    // let duration = start.elapsed();
+                    // println!("Total add_text: {:?}", duration);
                 }
 
                 content.clear();
@@ -411,7 +387,10 @@ impl Renderer {
             // Render last column and break row
             if column == (columns - 1) {
                 if !content.is_empty() {
+                    // let start = std::time::Instant::now();
                     content_builder.add_text(&content, last_style);
+                    // let duration = start.elapsed();
+                    // println!("Total add_text: {:?}", duration);
                 }
 
                 break;
@@ -421,18 +400,6 @@ impl Renderer {
         content_builder.finish_line();
     }
 
-    #[inline]
-    #[cfg(use_wa)]
-    pub fn decrease_foreground_opacity(&mut self, _acc: f32) {
-        // self.foreground_opacity -= acc;
-    }
-
-    #[inline]
-    #[cfg(use_wa)]
-    pub fn increase_foreground_opacity(&mut self, _acc: f32) {
-        // self.foreground_opacity += acc;
-    }
-
     #[inline]
     fn compute_color(&self, color: &AnsiColor, flags: Flags) -> ColorArray {
         match color {
@@ -610,7 +577,7 @@ impl Renderer {
         let mut style = FragmentStyle {
             color,
             background_color: Some(background_color),
-            font_attrs,
+            font_attrs: font_attrs.into(),
             ..FragmentStyle::default()
         };
 
@@ -620,20 +587,20 @@ impl Renderer {
             self.named_colors.vi_cursor
         };
 
-        let mut has_underline_cursor = false;
+        let (decoration, decoration_color) = self.compute_decoration(square);
+        style.decoration = decoration;
+        style.decoration_color = decoration_color;
 
         match self.cursor.state.content {
             CursorShape::Underline => {
                 style.decoration =
                     Some(FragmentStyleDecoration::Underline(UnderlineInfo {
-                        offset: -1.0,
-                        size: -1.0,
+                        offset: 0.0,
+                        size: 3.0,
                         is_doubled: false,
                         shape: UnderlineShape::Regular,
                     }));
                 style.decoration_color = Some(cursor_color);
-
-                has_underline_cursor = true;
             }
             CursorShape::Block => {
                 style.cursor = Some(SugarCursor::Block(cursor_color));
@@ -644,11 +611,6 @@ impl Renderer {
             CursorShape::Hidden => {}
         }
 
-        let (decoration, decoration_color) =
-            self.compute_decoration(square, has_underline_cursor);
-        style.decoration = decoration;
-        style.decoration_color = decoration_color;
-
         (style, content)
     }
 
@@ -714,6 +676,7 @@ impl Renderer {
 
         let mut content_builder = Content::builder();
 
+        // let start = std::time::Instant::now();
         for (i, row) in rows.iter().enumerate() {
             let has_cursor = is_cursor_visible && self.cursor.state.pos.row == i;
             self.create_line(
@@ -725,6 +688,8 @@ impl Renderer {
                 focused_match,
             );
         }
+        // let duration = start.elapsed();
+        // println!("Total loop rows: {:?}", duration);
 
         sugarloaf.set_content(content_builder.build());
 
