@@ -210,7 +210,7 @@ impl Screen<'_> {
                 && config.navigation.color_automation.is_empty()),
         };
         let context_manager = context::ContextManager::start(
-            (&renderer.get_cursor_state(), config.blinking_cursor),
+            (&renderer.get_cursor_state(), config.cursor.blinking),
             event_proxy,
             window_id,
             0,
@@ -330,7 +330,7 @@ impl Screen<'_> {
             let cursor = self.renderer.get_cursor_state_from_ref().content;
             terminal.cursor_shape = cursor;
             terminal.default_cursor_shape = cursor;
-            terminal.blinking_cursor = config.blinking_cursor;
+            terminal.blinking_cursor = config.cursor.blinking;
             drop(terminal);
         }
 
@@ -343,7 +343,7 @@ impl Screen<'_> {
             self.sugarloaf.set_background_image(image);
         }
 
-        self.demand_render();
+        self.render();
         self.resize_all_contexts();
     }
 
@@ -357,7 +357,7 @@ impl Screen<'_> {
 
         self.sugarloaf.update_font_size(action);
 
-        self.demand_render();
+        self.render();
         self.resize_all_contexts();
     }
 
@@ -383,7 +383,7 @@ impl Screen<'_> {
         self.render();
         self.resize_all_contexts();
 
-        self.demand_render();
+        self.render();
 
         self
     }
@@ -480,10 +480,8 @@ impl Screen<'_> {
                 // KEYBOARD_REPORT_ALL_KEYS_AS_ESC is used, we build proper escapes for
                 // the keys below.
                 _ if mode.contains(Mode::KEYBOARD_REPORT_ALL_KEYS_AS_ESC) => {
-                    crate::bindings::kitty_keyboard_protocol::build_key_sequence(
-                        key, mods, mode,
-                    )
-                    .into()
+                    crate::bindings::kitty_keyboard::build_key_sequence(key, mods, mode)
+                        .into()
                 }
                 // Winit uses different keys for `Backspace` so we explicitly specify the
                 // values, instead of using what was passed to us from it.
@@ -491,10 +489,8 @@ impl Screen<'_> {
                 Key::Named(NamedKey::Enter) => [b'\r'].as_slice().into(),
                 Key::Named(NamedKey::Backspace) => [b'\x7f'].as_slice().into(),
                 Key::Named(NamedKey::Escape) => [b'\x1b'].as_slice().into(),
-                _ => crate::bindings::kitty_keyboard_protocol::build_key_sequence(
-                    key, mods, mode,
-                )
-                .into(),
+                _ => crate::bindings::kitty_keyboard::build_key_sequence(key, mods, mode)
+                    .into(),
             };
 
             self.sugarloaf.mark_dirty();
@@ -517,7 +513,7 @@ impl Screen<'_> {
                 self.search_input(character);
             }
 
-            self.demand_render();
+            self.render();
             return;
         }
 
@@ -560,9 +556,7 @@ impl Screen<'_> {
                 bytes
             } else {
                 // Otherwise we should build the key sequence for the given input.
-                crate::bindings::kitty_keyboard_protocol::build_key_sequence(
-                    key, mods, mode,
-                )
+                crate::bindings::kitty_keyboard::build_key_sequence(key, mods, mode)
             }
         };
 
@@ -679,52 +673,52 @@ impl Screen<'_> {
                     Act::SearchForward => {
                         self.start_search(Direction::Right);
                         self.resize_top_or_bottom_line(self.ctx().len());
-                        self.demand_render();
+                        self.render();
                     }
                     Act::SearchBackward => {
                         self.start_search(Direction::Left);
                         self.resize_top_or_bottom_line(self.ctx().len());
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Search(SearchAction::SearchConfirm) => {
                         self.confirm_search();
                         self.resize_top_or_bottom_line(self.ctx().len());
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Search(SearchAction::SearchCancel) => {
                         self.cancel_search();
                         self.resize_top_or_bottom_line(self.ctx().len());
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Search(SearchAction::SearchClear) => {
                         let direction = self.search_state.direction;
                         self.cancel_search();
                         self.start_search(direction);
                         self.resize_top_or_bottom_line(self.ctx().len());
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Search(SearchAction::SearchFocusNext) => {
                         self.advance_search_origin(self.search_state.direction);
                         self.resize_top_or_bottom_line(self.ctx().len());
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Search(SearchAction::SearchFocusPrevious) => {
                         let direction = self.search_state.direction.opposite();
                         self.advance_search_origin(direction);
                         self.resize_top_or_bottom_line(self.ctx().len());
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Search(SearchAction::SearchDeleteWord) => {
                         self.search_pop_word();
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Search(SearchAction::SearchHistoryPrevious) => {
                         self.search_history_previous();
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Search(SearchAction::SearchHistoryNext) => {
                         self.search_history_next();
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ToggleViMode => {
                         let mut terminal =
@@ -733,7 +727,7 @@ impl Screen<'_> {
                         let has_vi_mode_enabled = terminal.mode().contains(Mode::VI);
                         drop(terminal);
                         self.renderer.set_vi_mode(has_vi_mode_enabled);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ViMotion(motion) => {
                         let mut terminal =
@@ -746,7 +740,7 @@ impl Screen<'_> {
                             self.renderer.set_selection(selection.to_range(&terminal));
                         };
                         drop(terminal);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Vi(ViAction::CenterAroundViCursor) => {
                         let mut terminal =
@@ -762,19 +756,19 @@ impl Screen<'_> {
                     }
                     Act::Vi(ViAction::ToggleNormalSelection) => {
                         self.toggle_selection(SelectionType::Simple, Side::Left);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Vi(ViAction::ToggleLineSelection) => {
                         self.toggle_selection(SelectionType::Lines, Side::Left);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Vi(ViAction::ToggleBlockSelection) => {
                         self.toggle_selection(SelectionType::Block, Side::Left);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Vi(ViAction::ToggleSemanticSelection) => {
                         self.toggle_selection(SelectionType::Semantic, Side::Left);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ConfigEditor => {
                         self.context_manager.switch_to_settings();
@@ -798,7 +792,7 @@ impl Screen<'_> {
                         let num_tabs = self.ctx().len();
                         self.cancel_search();
                         self.resize_top_or_bottom_line(num_tabs);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::TabCloseCurrent => {
                         self.clear_selection();
@@ -811,7 +805,7 @@ impl Screen<'_> {
 
                         let num_tabs = self.ctx().len().wrapping_sub(1);
                         self.resize_top_or_bottom_line(num_tabs);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::TabCloseUnfocused => {
                         self.clear_selection();
@@ -821,7 +815,7 @@ impl Screen<'_> {
                         }
                         self.context_manager.close_unfocused_tabs();
                         self.resize_top_or_bottom_line(1);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Quit => {
                         self.context_manager.quit();
@@ -844,7 +838,7 @@ impl Screen<'_> {
                             terminal.vi_mode_cursor.scroll(&terminal, scroll_lines);
                         terminal.scroll_display(Scroll::PageUp);
                         drop(terminal);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ScrollPageDown => {
                         // Move vi mode cursor.
@@ -857,7 +851,7 @@ impl Screen<'_> {
 
                         terminal.scroll_display(Scroll::PageDown);
                         drop(terminal);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ScrollHalfPageUp => {
                         // Move vi mode cursor.
@@ -870,7 +864,7 @@ impl Screen<'_> {
 
                         terminal.scroll_display(Scroll::Delta(scroll_lines));
                         drop(terminal);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ScrollHalfPageDown => {
                         // Move vi mode cursor.
@@ -883,7 +877,7 @@ impl Screen<'_> {
 
                         terminal.scroll_display(Scroll::Delta(scroll_lines));
                         drop(terminal);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ScrollToTop => {
                         let mut terminal =
@@ -894,7 +888,7 @@ impl Screen<'_> {
                         terminal.vi_mode_cursor.pos.row = topmost_line;
                         terminal.vi_motion(ViMotion::FirstOccupied);
                         drop(terminal);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ScrollToBottom => {
                         let mut terminal =
@@ -908,21 +902,21 @@ impl Screen<'_> {
                         terminal.vi_motion(ViMotion::FirstOccupied);
                         terminal.vi_motion(ViMotion::FirstOccupied);
                         drop(terminal);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::Scroll(delta) => {
                         let mut terminal =
                             self.context_manager.current_mut().terminal.lock();
                         terminal.scroll_display(Scroll::Delta(*delta));
                         drop(terminal);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ClearHistory => {
                         let mut terminal =
                             self.context_manager.current_mut().terminal.lock();
                         terminal.clear_saved_history();
                         drop(terminal);
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ToggleFullscreen => self.context_manager.toggle_full_screen(),
                     Act::Minimize => {
@@ -938,24 +932,24 @@ impl Screen<'_> {
                     Act::SelectTab(tab_index) => {
                         self.context_manager.select_tab(*tab_index);
                         self.cancel_search();
-                        self.demand_render();
+                        self.render();
                     }
                     Act::SelectLastTab => {
                         self.cancel_search();
                         self.context_manager.select_last_tab();
-                        self.demand_render();
+                        self.render();
                     }
                     Act::SelectNextTab => {
                         self.cancel_search();
                         self.clear_selection();
                         self.context_manager.switch_to_next();
-                        self.demand_render();
+                        self.render();
                     }
                     Act::SelectPrevTab => {
                         self.cancel_search();
                         self.clear_selection();
                         self.context_manager.switch_to_prev();
-                        self.demand_render();
+                        self.render();
                     }
                     Act::ReceiveChar | Act::None => (),
                     _ => (),
@@ -1901,14 +1895,9 @@ impl Screen<'_> {
     }
 
     #[inline]
-    fn demand_render(&mut self) {
-        self.update_content();
-        self.render();
-    }
-
-    pub fn update_content(&mut self) {
-        // let start = std::time::Instant::now();
-        // println!("Render time elapsed");
+    pub fn render(&mut self) {
+        // let start_total = std::time::Instant::now();
+        // println!("_____________________________\nrender time elapsed");
         let is_search_active = self.search_active();
         if is_search_active {
             if let Some(history_index) = self.search_state.history_index {
@@ -1930,6 +1919,7 @@ impl Screen<'_> {
             None
         };
 
+        // let start = std::time::Instant::now();
         let (rows, cursor, display_offset, has_blinking_enabled) = {
             let terminal = self.context_manager.current().terminal.lock();
             let data = (
@@ -1941,10 +1931,11 @@ impl Screen<'_> {
             drop(terminal);
             data
         };
-
-        self.context_manager.update_titles();
         self.renderer.set_ime(self.ime.preedit());
+        // let duration = start.elapsed();
+        // println!("Total terminal with set_ime info is: {:?}", duration);
 
+        // let start = std::time::Instant::now();
         self.renderer.prepare_term(
             &rows,
             cursor,
@@ -1955,16 +1946,14 @@ impl Screen<'_> {
             &mut search_hints,
             &self.search_state.focused_match,
         );
+
         // let duration = start.elapsed();
-        // println!("Total update_content is: {:?}\n", duration);
-    }
+        // println!("Total prepare_term is: {:?}", duration);
 
-    #[inline]
-    pub fn render(&mut self) {
         // let start = std::time::Instant::now();
-        // println!("Render time elapsed");
-
         self.sugarloaf.render();
+        // let duration = start.elapsed();
+        // println!("Total render is: {:?}", duration);
 
         // In this case the configuration of blinking cursor is enabled
         // and the terminal also have instructions of blinking enabled
@@ -1973,7 +1962,7 @@ impl Screen<'_> {
             self.context_manager.schedule_render_on_route(800);
         }
 
-        // let duration = start.elapsed();
-        // println!("Total render time is: {:?}\n", duration);
+        // let duration = start_total.elapsed();
+        // println!("Total whole render function is: {:?}\n", duration);
     }
 }
