@@ -7,6 +7,7 @@ pub mod renderer;
 pub mod theme;
 pub mod window;
 
+use crate::ansi::CursorShape;
 use crate::config::bindings::Bindings;
 use crate::config::defaults::*;
 use crate::config::keyboard::Keyboard;
@@ -69,8 +70,8 @@ impl Default for Developer {
 
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
 pub struct Config {
-    #[serde(default = "bool::default", rename = "blinking-cursor")]
-    pub blinking_cursor: bool,
+    #[serde(default)]
+    pub cursor: CursorConfig,
     #[serde(default = "Navigation::default")]
     pub navigation: Navigation,
     #[serde(default = "Window::default")]
@@ -103,8 +104,6 @@ pub struct Config {
     pub padding_x: f32,
     #[serde(rename = "padding-y", default = "default_padding_y")]
     pub padding_y: [f32; 2],
-    #[serde(default = "default_cursor")]
-    pub cursor: char,
     #[serde(default = "Vec::default", rename = "env-vars")]
     pub env_vars: Vec<String>,
     #[serde(default = "default_option_as_alt", rename = "option-as-alt")]
@@ -134,6 +133,14 @@ pub struct Config {
     pub renderer: Renderer,
 }
 
+#[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
+pub struct CursorConfig {
+    #[serde(default = "default_cursor")]
+    pub shape: CursorShape,
+    #[serde(default = "bool::default")]
+    pub blinking: bool,
+}
+
 #[cfg(not(target_os = "windows"))]
 #[inline]
 pub fn config_dir_path() -> PathBuf {
@@ -392,13 +399,12 @@ impl Config {
 impl Default for Config {
     fn default() -> Self {
         Config {
-            blinking_cursor: false,
+            cursor: CursorConfig::default(),
             editor: default_editor(),
             adaptive_theme: None,
             adaptive_colors: None,
             bindings: Bindings::default(),
             colors: Colors::default(),
-            cursor: default_cursor(),
             scroll: Scroll::default(),
             keyboard: Keyboard::default(),
             developer: Developer::default(),
@@ -422,6 +428,15 @@ impl Default for Config {
     }
 }
 
+impl Default for CursorConfig {
+    fn default() -> Self {
+        Self {
+            shape: default_cursor(),
+            blinking: false,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -461,7 +476,7 @@ mod tests {
     fn test_filepath_does_not_exist_with_fallback() {
         let config = Config::load_from_path(&tmp_dir().join("it-should-never-exist"));
         assert_eq!(config.theme, String::default());
-        assert_eq!(config.cursor, default_cursor());
+        assert_eq!(config.cursor.shape, default_cursor());
     }
 
     #[test]
@@ -500,9 +515,9 @@ mod tests {
         );
         let env_vars: Vec<String> = vec![];
         assert_eq!(result.env_vars, env_vars);
-        assert_eq!(result.cursor, default_cursor());
+        assert_eq!(result.cursor.shape, default_cursor());
         assert_eq!(result.theme, String::default());
-        assert_eq!(result.cursor, default_cursor());
+        assert_eq!(result.cursor.shape, default_cursor());
         assert_eq!(result.fonts, SugarloafFonts::default());
         assert_eq!(result.shell, default_shell());
         assert!(!result.renderer.disable_unfocused_render);
@@ -578,7 +593,7 @@ mod tests {
 
         assert_eq!(result.renderer.performance, renderer::Performance::High);
         assert_eq!(result.env_vars, [String::from("A=5"), String::from("B=8")]);
-        assert_eq!(result.cursor, default_cursor());
+        assert_eq!(result.cursor.shape, default_cursor());
         assert_eq!(result.fonts, SugarloafFonts::default());
         assert_eq!(result.theme, String::default());
         // Colors
@@ -601,13 +616,14 @@ mod tests {
         let result = create_temporary_config(
             "change-cursor",
             r#"
-            cursor = '_'
+            [cursor]
+            shape = 'underline'
         "#,
         );
 
         assert_eq!(result.renderer.performance, renderer::Performance::High);
         assert_eq!(result.renderer.backend, renderer::Backend::Automatic);
-        assert_eq!(result.cursor, '_');
+        assert_eq!(result.cursor.shape, CursorShape::Underline);
         assert_eq!(result.fonts, SugarloafFonts::default());
         assert_eq!(result.theme, String::default());
         // Colors
