@@ -1,10 +1,11 @@
 pub mod components;
 pub mod context;
 pub mod font;
+pub mod font_introspector;
 pub mod layout;
 mod sugarloaf;
 
-pub use swash::{Stretch, Style, Weight};
+pub use font_introspector::{Stretch, Style, Weight};
 
 pub use crate::sugarloaf::{
     compositors::SugarCompositors,
