@@ -121,7 +121,7 @@ impl Sugarloaf<'_> {
         let ctx = Context::new(window, renderer);
 
         let text_brush = {
-            let data = { &font_library.inner.read().unwrap().main };
+            let data = { &font_library.inner.read().unwrap().ui };
             text::GlyphBrushBuilder::using_fonts(vec![data.to_owned()])
                 .build(&ctx.device, ctx.format)
         };
@@ -333,9 +333,6 @@ impl Sugarloaf<'_> {
                         self.layer_brush.render(0, &mut rpass, None);
                     }
 
-                    self.rich_text_brush
-                        .render(&mut self.ctx, &self.state, &mut rpass);
-
                     if self.graphics.has_graphics_on_top_layer() {
                         let range_request = if self.graphics.bottom_layer.is_some() {
                             1..(self.graphics.top_layer.len() + 1)
@@ -347,6 +344,9 @@ impl Sugarloaf<'_> {
                         }
                     }
 
+                    self.rich_text_brush
+                        .render(&mut self.ctx, &self.state, &mut rpass);
+
                     self.rect_brush
                         .render(&mut rpass, &self.state, &mut self.ctx);
 
